package xmdv.dataproc;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



/**

 * An instance of this class defines an Multidimensional Brush.

 * Each Brush has a dimensionality N, and N Brush1Ds indexed from 0 to N-1.

 * 

 * @author  Jing Yang

 * @version v1.0

 */

public class Brush {

    // Variables declaration

    protected int brush1DNum = 0;

    protected Brush1D[] brushDims = null;

    // End of variables declaration

    

    /** Creates new Brush*/

    public Brush() {

    }

    

    /** Creates new Brush

     * @param brush1DNum the number of Brush1Ds to be included in this Brush, 

     * i.e., the dimensionality of this brush 

     */

    public Brush(int brush1DNum) {

        setBrush1DNum(brush1DNum);

    }

    

    /**

     * Returns a Brush that copys this Brush

     * @return a Brush that copys this Brush

     */

    public Brush copy() {

        Brush newBrush = new Brush(brush1DNum);

        for (int i = 0; i < brush1DNum; i++ ) {

            newBrush.setBrush1D(i, brushDims[i].copy());

        }

        

        return newBrush;

    }

    

    /** Sets the number of Brush1Ds to be included in this Brush, 

     * i.e., sets the dimensionality of this brush 

     * @param brush1DNum the number of Brush1Ds to be included in this Brush, 

     * i.e., the dimensionality of this brush 

     */

    public void setBrush1DNum(int brush1DNum) {

        this.brush1DNum = brush1DNum;

        brushDims = new Brush1D[brush1DNum];

        for (int i = 0; i < brush1DNum; i++ ) {

            brushDims[i] = new Brush1D();    

        }

    }

    

    /** Sets a Brush1D in this Brush, 

     * @param dimIndex the index of the Brush1D to be set in this Brush 

     * @param brushDim the Brush1D to be set

     * @return false if the index is out of range of this Brush, i.e., larger or equal to the 

     * dimensionality of this Brush, otherwise returns true

     */

    public boolean setBrush1D(int dimIndex, Brush1D brushDim) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return false;

        brushDims[dimIndex] = brushDim;

        return true;

    }

    

    /**

     * Set the maximum value of a Brush1D in this Brush

     * @param max the maximum value to be set. 

     * If max is larger than 1, the maximum value will be set to 1. 

     * If max is smaller than the minimum value of this Brush1D, the maximum value will be set to the minimum value. 

     * @param dimIndex the index of the Brush1D to be set in this Brush 

     */

    public void setNorMax(int dimIndex, double max) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return;

        brushDims[dimIndex].setMax(max);

    }

    

    /**

     * Set the minimum value of this Brush1D

     * @param min the minimum value to be set. 

     * If min is smaller than 0, the minimum value will be set to 0. 

     * If min is larger than the maximum value of this Brush1D, the minimum value will be set to the maximum value. 

     * @param dimIndex the index of the Brush1D to be set in this Brush 

     */

    public void setNorMin(int dimIndex, double min) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return;

        brushDims[dimIndex].setMin(min);

    }

    

    /**

     * Set the minimum value and the maximum value of this Brush1D

     * @param min the minimum value to be set. 

     * @param max the maximum value to be set. 

     * @param dimIndex the index of the Brush1D to be set in this Brush 

     */

    public void setNorMinMax(int dimIndex, double min, double max) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return;

        brushDims[dimIndex].setMinMax(min, max);

    }

    

    /**

     * Gets the dimensionality of this Brush.  

     * @return the dimensionality of this Brush

     */

    public int getBrush1DNum() {

        return brush1DNum;

    }

    

    /**

     * Gets a Brush1D in this Brush.  

     * @param dimIndex the index of the Brush1D to be set in this Brush 

     * @return the indexed Brush1D. Returns null if the index if out of range

     */

    public Brush1D getBrush1D(int dimIndex) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return null;

        return brushDims[dimIndex];

    }

    

    /**

     * Gets the maximum value of a Brush1D in this Brush.  

     * @param dimIndex the index of the Brush1D

     * @return the maximum value of the indexed Brush1D. Returns -1 if the index is out of range

     */

    public double getNorMax(int dimIndex) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return -1;

        return brushDims[dimIndex].getMax();

    }

    

    /**

     * Gets the minimum value of a Brush1D in this Brush.  

     * @param dimIndex the index of the Brush1D

     * @return the minimum value of the indexed Brush1D. Returns -1 if the index is out of range

     */

    public double getNorMin(int dimIndex) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return -1;

        return brushDims[dimIndex].getMin();

    }

    

    /**

     * Gets the center of a Brush1D in this Brush.  

     * @param dimIndex the index of the Brush1D

     * @return the center of the indexed Brush1D. Returns -1 if the index is out of range

     */

    public double getNorCenter(int dimIndex) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return -1;

        return brushDims[dimIndex].getCenter();

    }

    

    /**

     * Gets the radius of a Brush1D in this Brush.  

     * @param dimIndex the index of the Brush1D

     * @return the radius of the indexed Brush1D. Returns -1 if the index is out of range

     */

    public double getNorRadius(int dimIndex) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return -1;

        return brushDims[dimIndex].getRadius();

    }

    

    /**

     * Shifts a Brush1D in this Brush.  

     * @param dimIndex the index of the Brush1D

     * @param shiftValue value to be added to the maximum and minimum value of the indexed Brush1D.

     */

    public void shift(int dimIndex, double shiftValue) {

        if (dimIndex < 0 || dimIndex > brush1DNum - 1)

            return;

        brushDims[dimIndex].shift(shiftValue);

    }

    

    /**

     * Judges if a multidimensional data is within the range of this Brush.  

     * @param norData a multidimensional data

     * @return true if the data is within the Brush. Otherwise returns false..

     */

    public boolean isDataWithinBrush(double[] norData) {

        if (norData.length != brush1DNum)

            return false;

        

        boolean in = true;

        for (int i = 0; i < brush1DNum; i++ ) {

            if (norData[i] > getNorMax(i) || norData[i] < getNorMin(i)) {

                in = false;

                break;

            }

        }

        

        return in;

    }

}