package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import java.awt.geom.Point2D;

import java.lang.Math;

import xmdv.dataproc.*;



/**

 * This class is an implementation of the abstract class FlatDisplay using 

 * the Scatterplot Matrix technique.

 *

 * @author  Jing Yang

 * @version v1.0

 */

public class FlatScatter extends FlatDisplay{

    private Point2D.Double prevPosition;

    private int[] prevPlotIndex;

    private double[] prevNorValue;

    private Point2D.Double[] pts;

    private double[] xAxesPos;

    private double[] yAxesPos;

    private double[] xAxesInterval;

    private double[] yAxesInterval;

    

    private int[] plotIndex;

    private boolean[] resizeMin;

    

    private double multiselRadius = 0.01;

    

    /* inits: pt = point, pts = points, lo = low, hi = high, num = number 

     *        nor = normalized, prev = previous, plotIndex = dimension 

     *        min = minimum, max = maximum */

        

    /** Creates a new instance of FlatScatter display */

    public FlatScatter () {

    }

    

    public FlatScatter(grCanvas canvas, ColorManager colorManager, Dataset dataset) {

        setCanvas(canvas);

        setColorManager(colorManager);

        setDataset(dataset);

    }

    

    protected boolean isDimensionalityValid()  

    {

        if ( dataset.getCurrDimNum() < 1 ) {

            System.out.println("No sufficent dimensions for this kind of display.");

            return false;

        } else 

            return true;

    }

    

    protected void setParameters() {

        super.setParameters();

        

        if (dimNum != 0) {

            int i;

            double interval = 1. / dimNum;

            xAxesPos = new double[dimNum];

            xAxesInterval = new double[dimNum];

            for (i = 0; i < dimNum; i++) {

                xAxesInterval[i] = interval;

                xAxesPos[i] = interval * i;

            }

     

            yAxesPos = new double[dimNum];

            yAxesInterval = new double[dimNum];

            for (i = 0; i < dimNum; i++) {

                yAxesInterval[i] = interval;

                yAxesPos[dimNum - 1 - i] = interval * i;

            }

            

        }    

        

        plotIndex = new int[2];

        resizeMin = new boolean[2];

    }

    

    

    protected void drawSingleData(double[] norData, int dataIndex) {

        canvas.setPointSize(2.0);

    

        Point2D.Double pt;

        for ( int i = 0; i < dimNum; i++ ) {

            for ( int j = 0; j < dimNum; j++ )

            {

                pt = getPosition(i, j, norData[i], norData[j]);

                canvas.drawPoint(pt);

            }

        }

    }

    

    protected void drawSingleBrush(Brush brush) {

        canvas.setForegroundColor(colorManager.getColor("brush"));

    

        for (int i = 0; i < dimNum; i++) {

            for(int j = 0; j < dimNum; j++) {

                Point2D.Double pt1, pt2;

                

                pt1 = getPosition(i, j, brush.getNorMax(i), brush.getNorMax(j));

                pt2 = getPosition(i, j, brush.getNorMin(i), brush.getNorMin(j));

    

                canvas.fillRectangle(pt1, pt2);

            }

        }

    }

    

    protected void drawSingleBrushBox(Brush brush, int xIndex, int yIndex) {

        int i;

        Point2D.Double pt1, pt2;

        

        canvas.setForegroundColor(colorManager.getColor("brushbox"));

        canvas.setLineWidth(1.0);

        for (i = 0; i < dimNum; i++) {

            pt1 = getPosition(xIndex, i, brush.getNorMax(xIndex), brush.getNorMax(i));

            pt2 = getPosition(xIndex, i, brush.getNorMin(xIndex), brush.getNorMin(i));

    

            canvas.drawRectangle(pt1, pt2);

        }

    

        for (i = 0; i < dimNum; i++) {

            pt1 = getPosition(i, yIndex, brush.getNorMax(i), brush.getNorMax(yIndex));

            pt2 = getPosition(i, yIndex, brush.getNorMin(i), brush.getNorMin(yIndex));

    

            canvas.drawRectangle(pt1, pt2);

        }

    }

    

    protected void drawAxes()

    {

        canvas.setLineWidth(1.);

    

        // Draw the vertical axes

        int i;

        Point2D.Double pt1, pt2;

        for (i = 0; i < dimNum; i++) {

            canvas.setForegroundColor(colorManager.getColor("grid0"));

            pt1 = getPosition(i, 0, 0, 1);

            pt2 = getPosition(i, dimNum - 1, 0, 0);

            canvas.drawLine(pt1, pt2);

        }

        pt1 = getPosition(dimNum - 1, 0, 1, 1);

        pt2 = getPosition(dimNum - 1, dimNum - 1, 1, 0);

        canvas.drawLine(pt1, pt2);

    

        // Draw the horizontal axes

        int j;

        for (j = 0; j < dimNum; j++) {

            canvas.setForegroundColor(colorManager.getColor("grid0"));

            pt1 = getPosition(0, j, 0, 1);

            pt2 = getPosition(dimNum - 1, j, 1, 1);

            canvas.drawLine(pt1, pt2);

        }

        pt1 = getPosition(0, dimNum - 1, 0, 0);

        pt2 = getPosition(dimNum - 1, dimNum - 1, 1, 0);

        canvas.drawLine(pt1, pt2);

    

        //Draw axes names and mins and maxes

        DrawScatterAxesName(canvas);

    }

    

    protected void inputMouseMotion(Point2D.Double pt)

    {

        if (inputMode == CANVAS_PAN) {

            canvas.pan(prevPosition, pt);

            canvas.refresh();

        } else if (inputMode == NONE) {

            if (! getPlot(pt, plotIndex))

                return;

            double[] norValue = new double[2];

            if (!getNorValue(pt, plotIndex, norValue)) {

                return;

            }

            scatterCaption(plotIndex, norValue);

        }   else {

            double[] norValue = new double[2];

            if (!getNorValue(pt, plotIndex, norValue)) {

                return;

            }

            scatterCaption(plotIndex, norValue);

        

            switch (inputMode) {

                case RESIZE_BRUSH:

                    if (xor_needs_undo) 

                        inputScatterResizeBrush(prevPlotIndex, prevNorValue);

                    inputScatterResizeBrush(plotIndex, norValue);

                    dataset.refreshAllCanvas();

                    break;

        

                case TRANSLATE_BRUSH:

                    if (xor_needs_undo) 

                        inputScatterMoveBrush(prevPlotIndex, prevNorValue);

                    inputScatterMoveBrush(plotIndex, norValue);

                    dataset.refreshAllCanvas();

                    break;

        

                case MULTI_SELECT:

                    inputScatterMultiSelect(plotIndex, norValue);

                    dataset.refreshAllCanvas();

                    return;

        

                default: break;

            }

            prevPosition = pt;

            prevPlotIndex = plotIndex;

            prevNorValue = norValue;

        }    

    }

    

    

    protected void inputButtonPress(int button, Point2D.Double pt) {

        inputMode = NONE;

        prevPosition = pt;

    

        if ((!canvas.altkey) && (!canvas.controlkey) && button == 3) {

            inputMode = CANVAS_PAN;

            canvas.setMoveCursor();

            return;

        }  

    

        if (! getPlot(pt, plotIndex))

            return;

        double[] norValue = new double[2];

        getNorValue(pt, plotIndex, norValue);

    

        if (canvas.shiftkey && button == 1) {

            inputMode = MULTI_SELECT;

            inputScatterMultiSelect(plotIndex,norValue);

            return;

        } 

    

        if (button == 1) {

            inputMode = RESIZE_BRUSH;

            for ( int i = 0; i < 2; i++ ) {

                double norMin = brush.getNorMin(plotIndex[i]);

                double norMax = brush.getNorMax(plotIndex[i]);

                if (Math.abs(norMax - norValue[i]) < Math.abs(norValue[i] - norMin) ) {

                    resizeMin[i] = false;

                } else {

                    resizeMin[i] = true;

                }            

            }

        } else if (button == 2) {

            inputMode = TRANSLATE_BRUSH;

        }

    

        inputMouseMotion(pt);

    }

    

    

    protected void inputButtonRelease(int button, Point2D.Double pt)

    {

        switch (inputMode) {

    

            case CANVAS_PAN:

                canvas.setDefaultCursor();

                break;

    

            case MULTI_SELECT:

                break;

    

            case RESIZE_BRUSH:

            case TRANSLATE_BRUSH:

                dataset.brush();

                canvas.setDefaultCursor();

                dataset.refreshAllCanvas();

                break;

    

            default: break;

        }

    

        inputMode = NONE;

        xor_needs_undo = false;

    }

    

    protected Point2D.Double getPosition(int i, int j, double xValue, double yValue)

    {

        return new Point2D.Double(xAxesPos[i] + xValue * xAxesInterval[i], yAxesPos[j] + yValue * yAxesInterval[j]);

    }

    

    

    protected boolean getPlot(Point2D.Double pt, int[] plotIndex)

    {

        int i;

        for (i = 0; i < 2; i++ )

            plotIndex[i] = -1;    

        double x = pt.getX();

        double y = pt.getY();

        

        if ( y > 0. && y < 1.0 && x > 0. && x < 1.0 ) {

            plotIndex[0] = dimNum-1;

            for ( i = 0; i < dimNum-1; i++ ) {

                if ( x < xAxesPos[i] + xAxesInterval[i] )

                {

                    plotIndex[0] = i;

                    break;

                }

            }

    

            plotIndex[1] = 0;

            for ( i = dimNum - 1; i > 0; i-- ) {

                if ( y < yAxesPos[i] + yAxesInterval[i] )

                {

                    plotIndex[1] = i;

                    break;

                }

            }

        }

        if ( plotIndex[0] < 0 || plotIndex[1] < 0 )

            return false;

        else

            return true;

    }

    

    protected boolean getNorValue(Point2D.Double pt, int[] plotIndex, double[] norValue) {

        int i;

        

        for (i = 0; i < 2; i++) {

            if (plotIndex[i] < 0 || plotIndex[i] >= dimNum)

                return false;

        }        

        

        norValue[0] = (pt.getX() - xAxesPos[plotIndex[0]]) / xAxesInterval[plotIndex[0]];

        norValue[1] = (pt.getY() - yAxesPos[plotIndex[1]]) / yAxesInterval[plotIndex[1]];

        

        for (i = 0; i < 2; i++) {

            if (norValue[i] < 0)

                norValue[i] = 0;

            else if (norValue[i] > 1)

                norValue[i] = 1;

        }

        

        if (plotIndex[0] == plotIndex[1]) {

            norValue[0] = norValue[1] = (norValue[0] + norValue[1]) / 2;

        }        

        return true;

    }

    

    

    protected void DrawScatterAxesName(grCanvas canvas)

    {

        canvas.setForegroundColor(colorManager.getColor("text"));

    

        double x, y;

        Point2D.Double pt;

         

        double fontWidth = canvas.getFontWidth('W');

        double fontHeight = canvas.getFontHeight('H');

        double widthPad = fontWidth * 0.1;

        double heightPad = fontHeight * 0.1;

        for (int i = 0; i < dimNum; i++)

        {

            String dimName = dims[i].getName();

    

            pt = getPosition(i, 0, 0, 1);

            y = pt.getY() + heightPad + fontHeight;

            x = pt.getX() + widthPad;       

            canvas.drawHoriString(new Point2D.Double(x, y),

                                  xAxesInterval[i] - widthPad, dimName);

    

            pt = getPosition(0, i, 0, 1);

            x = pt.getX() - widthPad - fontHeight;

            y = pt.getY() - heightPad;       

            canvas.drawVertString(new Point2D.Double(x, y),

                                  yAxesInterval[i] - heightPad, dimName);

        }

    }

    

    

    protected void inputScatterResizeBrush(int[] plotIndex, double[] norValue)

    {

        // If this is a diagonal plot (same dimension on both axes)

        // then average the x and y positions

        //

        if (plotIndex[0] == plotIndex[1]) {

            norValue[0] = (norValue[0] + norValue[1])/2;

            norValue[1] = norValue[0];

        }

        for ( int i = 0; i < 2; i++ ) {

            if (resizeMin[i]) {

                brush.setNorMin(plotIndex[i], norValue[i]);

            } else {

                brush.setNorMax(plotIndex[i], norValue[i]);

            }            

        }

        canvas.setQuickDraw(true);

        drawSingleBrushBox(brush, plotIndex[0], plotIndex[1]);

        canvas.setQuickDraw(false);

        xor_needs_undo = true;

    }

    

    

    protected void inputScatterMoveBrush(int[] plotIndex, double[] norValue)

    {

        Brush tempBrush = brush.copy();

        for (int i = 0; i < 2; i++) {

            tempBrush.shift(plotIndex[i], norValue[i] - brush.getNorCenter(plotIndex[i]));

        }

        canvas.setQuickDraw(true);

        drawSingleBrushBox(brush, plotIndex[0], plotIndex[1]);

        canvas.setQuickDraw(false);

    

        for (int i = 0; i < 2; i++) {

            brush.shift(plotIndex[i], norValue[i] - brush.getNorCenter(plotIndex[i]));

        }

        xor_needs_undo = true;

    }

    

    protected void inputScatterMultiSelect(int[] plotIndex, double[] norValue)

    {

        for (int i = 0; i < dataNum; i++)

        {

            double[] norData = dataset.getCurrNorData(i);

    

            if (Math.abs(norData[plotIndex[0]] - norValue[0]) < multiselRadius &&

                Math.abs(norData[plotIndex[1]] - norValue[1]) < multiselRadius )

            {

                dataset.multiSelectAddPoint(i);

                setQuickDrawCase0(norData, i, colorManager.getColor("paint"));

                canvas.paintImmediately(0, 0, canvas.getWidth(), canvas.getHeight()); 

            }

        }

    }

    

    protected void scatterCaption(int[] plotIndex, double[] norValue)

    {

        String valueMessage;

        if (plotIndex[0] != plotIndex[1]) {

            valueMessage = dims[plotIndex[0]].getNamedValueMessage(norValue[0]) + ", "

                            + dims[plotIndex[1]].getNamedValueMessage(norValue[1]);

        } else {

            valueMessage = dims[plotIndex[0]].getNamedValueMessage(norValue[0]);

        }        

    

        canvas.showMessage(valueMessage);

    }

    

    public void quickDraw() {

        if (quickDrawCase == 0) {

            canvas.setForegroundColor(quickDrawColor);

            drawSingleData(quickDrawNorData, quickDrawDataIndex);

        } else if (quickDrawCase == 1) {

        

        }

        

    }

}

